@@ -65,6 +65,38 @@ static void command_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const ch
 
 static gboolean get_file_charset (const gchar * filename, gchar *charset, gint len);
 
+static void
+send_path_changed_event (GtkHTMLControlData *cd)
+{
+	if (cd && cd->editor_bonobo_engine) {
+		GNOME_GtkHTML_Editor_Engine engine;
+		GNOME_GtkHTML_Editor_Listener listener;
+		CORBA_Environment gev;
+
+		CORBA_exception_init (&gev);
+		engine = bonobo_object_corba_objref (BONOBO_OBJECT (cd->editor_bonobo_engine));
+
+		if (engine != CORBA_OBJECT_NIL
+		    && (listener = GNOME_GtkHTML_Editor_Engine__get_listener (engine, &gev)) != CORBA_OBJECT_NIL) {
+			CORBA_Environment ev;
+			CORBA_any *any;
+			CORBA_any *retval;
+
+			any = CORBA_any_alloc ();
+			any->_type = TC_null;
+			CORBA_exception_init (&ev);
+			retval = GNOME_GtkHTML_Editor_Listener_event (listener, "file_path_changed", any, &ev);
+			if (!BONOBO_EX (&ev))
+				CORBA_free (retval);
+
+			CORBA_exception_free (&ev);
+			CORBA_free (any);
+		}
+
+		CORBA_exception_free (&gev);
+	}
+}
+
 static void
 paste_quotation_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
@@ -102,6 +134,9 @@ insert_image_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cna
 					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 					       NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
+
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filesel), cd->file_path);
+
 	if (filesel) {
 		if (gtk_dialog_run (GTK_DIALOG (filesel)) == GTK_RESPONSE_OK) {
 			char *url = NULL;
@@ -111,6 +146,18 @@ insert_image_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cna
 					      NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
 			html_engine_paste_object (cd->html->engine, img, 1);
 			g_free (url);
+
+			url = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (filesel));
+
+			if (url) {
+				gboolean changed = strcmp (cd->file_path, url) != 0;
+
+				g_free (cd->file_path);
+				cd->file_path = url;
+
+				if (changed)
+					send_path_changed_event (cd);
+			}
 		}
 		gtk_widget_destroy (filesel);
 	}
@@ -212,15 +259,15 @@ file_dialog_ok (GtkWidget *w, GtkHTMLControlData *cd)
 
 	if (error || !io)
 		goto end;
-		
+
 	/* try reading as utf-8 */
 	g_io_channel_read_to_end (io, &data, &len, &error);
 
 	/* If we get a charset error try reading as the locale charset */
 	if (error && g_error_matches (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE)) {
 
-		/* 
-		 * Close the io channel since we can't set the 
+		/*
+		 * Close the io channel since we can't set the
 		 * charset once we've begun reading.
 		 */
 		g_io_channel_unref (io);
@@ -232,7 +279,7 @@ file_dialog_ok (GtkWidget *w, GtkHTMLControlData *cd)
 			error = NULL;
 
 			io = g_io_channel_new_file (filename, "r", &error);
-		
+
 			if (error || !io)
 				goto end;
 
@@ -240,14 +287,14 @@ file_dialog_ok (GtkWidget *w, GtkHTMLControlData *cd)
 			g_io_channel_read_to_end (io, &data, &len, &error);
 		}
 	}
-	
+
 	if (error)
 		goto end;
-	
+
 	if (cd->file_html) {
 		GtkHTML *tmp;
 		GtkHTMLStream *stream;
-		
+
 		tmp = GTK_HTML (gtk_html_new ());
 		stream = gtk_html_begin_content (tmp, "text/html; charset=utf-8");
 		gtk_html_write (tmp, stream, data, len);
@@ -264,9 +311,9 @@ file_dialog_ok (GtkWidget *w, GtkHTMLControlData *cd)
 
 	if (error) {
 		GtkWidget *toplevel;
-		
+
 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (cd->html));
-		
+
 		if (GTK_WIDGET_TOPLEVEL (toplevel)) {
 			GtkWidget *dialog;
 
@@ -276,7 +323,7 @@ file_dialog_ok (GtkWidget *w, GtkHTMLControlData *cd)
 							 GTK_BUTTONS_CLOSE,
 							 _("Error loading file '%s': %s"),
 							 filename, error->message);
-		
+
 			/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
 			g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
 						  G_CALLBACK (gtk_widget_destroy),
@@ -306,11 +353,26 @@ insert_file_dialog (GtkHTMLControlData *cd, gboolean html)
 						       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 						       NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (cd->file_dialog), GTK_RESPONSE_OK);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cd->file_dialog), g_get_home_dir ());
+
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cd->file_dialog), cd->file_path);
 
 	if (cd->file_dialog) {
 		if (gtk_dialog_run (GTK_DIALOG (cd->file_dialog)) == GTK_RESPONSE_OK) {
+			gchar *path;
+
 			file_dialog_ok (cd->file_dialog, cd);
+
+			path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (cd->file_dialog));
+
+			if (path) {
+				gboolean changed = strcmp (cd->file_path, path) != 0;
+
+				g_free (cd->file_path);
+				cd->file_path = path;
+
+				if (changed)
+					send_path_changed_event (cd);
+			}
 		}
 		gtk_widget_destroy (cd->file_dialog);
 		cd->file_dialog = NULL;
@@ -329,7 +391,7 @@ insert_html_file_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char
 	insert_file_dialog (cd, TRUE);
 }
 
-static void 
+static void
 spell_check_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
 	spell_check_dialog (cd, TRUE);
@@ -512,7 +574,7 @@ static struct {
 static struct {
 	GtkHTMLFontStyle style;
 	const gchar *verb;
-} font_style_assoc[] = {	
+} font_style_assoc[] = {
 	{GTK_HTML_FONT_STYLE_FIXED, "FormatFixed"},
 	{GTK_HTML_FONT_STYLE_SUBSCRIPT, "FormatSubscript"},
 	{GTK_HTML_FONT_STYLE_SUBSCRIPT, "FormatSuperscript"},
@@ -523,7 +585,7 @@ static void
 font_style_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
         int i;
-                                                                              
+
         /* g_warning ("wowee %s :: %s", path, state); */
         for (i = 0; font_style_assoc[i].verb != NULL; i++) {
                 if (!strcmp (cname, font_style_assoc[i].verb))
@@ -531,14 +593,14 @@ font_style_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname
         }
 }
 
-static void 
+static void
 command_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
 	int i;
-	
+
 	for (i = 0; command_assoc[i].verb != NULL; i++) {
 		if (!strcmp (cname, command_assoc[i].verb)) {
-			gtk_html_command (cd->html, command_assoc[i].command); 
+			gtk_html_command (cd->html, command_assoc[i].command);
 			return;
 		}
 	}
@@ -603,9 +665,9 @@ menubar_update_format (GtkHTMLControlData *cd)
 					      "sensitive", sensitive, &ev);
 
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingH1",
-					      "sensitive", sensitive, &ev);	
+					      "sensitive", sensitive, &ev);
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingH2",
-					      "sensitive", sensitive, &ev);	
+					      "sensitive", sensitive, &ev);
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingH3",
 					      "sensitive", sensitive, &ev);
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingH4",
@@ -617,7 +679,7 @@ menubar_update_format (GtkHTMLControlData *cd)
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingAddress",
 					      "sensitive", sensitive, &ev);
 
-		bonobo_ui_component_thaw (uic, &ev);	
+		bonobo_ui_component_thaw (uic, &ev);
 
 		CORBA_exception_free (&ev);
 	}
@@ -878,13 +940,13 @@ menubar_setup (BonoboUIComponent  *uic,
 	}
 }
 
-gboolean 
+gboolean
 get_file_charset (const gchar * filename, gchar *charset, gint len)
 {
 	char data[1024];
 	gboolean result = FALSE;
 	FILE *fp = fopen (filename, "r");
-	
+
 	if (!fp)
 		return FALSE;
 
